from flask import Flask, json

from kw.json.flask import JSONExtension

items = {"secret": "foo"}.items()


def test_mask_dict():
    app = Flask(__name__)
    extension = JSONExtension(app)
    assert extension.app is app
    with app.app_context():
        assert json.dumps(items) == '{"secret": "foo"}'


def test_create_extension_without_app():
    assert JSONExtension().app is None



import datetime
import os
import sys
import uuid
from collections import namedtuple
from decimal import Decimal
from functools import partial
from json import dumps as json_dumps
from json import load as json_load
from json import loads

import arrow
import asyncio
import asyncpg
import attr
import pytest
from pytz import UTC
from sqlalchemy import Column, Integer, String, create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from kw.json import (
    KiwiJSONEncoder,
    MaskedJSONEncoder,
    default_encoder,
    dump,
    dumps,
    raw_encoder,
)
from kw.json._compat import DataclassItem, enum
from kw.json.exceptions import KiwiJsonError

try:
    from simplejson import dumps as simplejson_dumps
except ImportError:
    simplejson_dumps = None


Base = declarative_base()


class User(Base):
    __tablename__ = "users"
    id = Column(Integer, primary_key=True)
    name = Column(String)

    def asdict(self):
        return {"id": self.id, "name": self.name}


class Custom:
    pass


Namedtuple = namedtuple("Namedtuple", ["a", "b"])
test_namedtuple = Namedtuple(1.3333, 2.3333)
test_namedtuple_complex = Namedtuple(1.3333, Namedtuple(1.3333, {1: 1.3333}))


@attr.s
class AttrsItem(object):
    attrib = attr.ib(type=int)


class NotDataclassesItem(object):
    __dataclass_fields__ = {}


class NotAttrsItem(object):
    __attrs_attrs__ = ()


class HTML:
    def __html__(self):
        return "foo"


UUID = uuid.uuid4()

items_view = {"foo": 1}.items()
items_view_float = {"foo": 1.333}.items()
items_view_complex = {1: 1.333, 2: {2: 0.333}.items()}.items()


@pytest.mark.parametrize(
    "value, expected, date_as_unix_time",
    (
        ({1}, [1], False),
        (Decimal("1"), "1", False),
        (UUID, str(UUID), False),
        (datetime.datetime(2018, 1, 1), "2018-01-01T00:00:00", False),
        (datetime.datetime(2018, 1, 1, tzinfo=UTC), "2018-01-01T00:00:00+00:00", False),
        (arrow.get("2018-01-01"), "2018-01-01T00:00:00+00:00", False),
        (datetime.date(2018, 1, 1), "2018-01-01", False),
        (datetime.datetime(2018, 1, 1), 1514764800, True),
        (datetime.datetime(2018, 1, 1, tzinfo=UTC), 1514764800, True),
        (datetime.date(2018, 1, 1), 1514764800, True),
        (arrow.get("2018-01-01"), 1514764800, True),
        (HTML(), "foo", False),
        (items_view, {"foo": 1}, False),
    ),
)
def test_default_encoder(value, expected, date_as_unix_time):
    assert default_encoder(value, date_as_unix_time=date_as_unix_time) == expected


@pytest.mark.skipif(
    simplejson_dumps is None, reason="Decimal encoding with simplejson only"
)
@pytest.mark.parametrize(
    "value, expected",
    (
        (Decimal("1"), "1"),
        (Decimal("-1"), "-1"),
        (Decimal("0.123456789123456789"), "0.123456789123456789"),
    ),
)
def test_simplejson_encoder_with_decimal(value, expected):
    assert dumps(value, use_decimal=True) == expected


@pytest.mark.skipif(simplejson_dumps is not None, reason="Standard json only")
def test_json_with_use_decimal_argument():
    with pytest.raises(KiwiJsonError, match=r".*Decimal.*"):
        dumps(Decimal(0), use_decimal=True)


def test_default_encoder_defaults():
    # By default `default_encoder` encodes datetimes as ISO
    assert default_encoder(datetime.datetime(2018, 1, 1)) == "2018-01-01T00:00:00"


@pytest.mark.parametrize(
    "value, expected, date_as_unix_time",
    (
        ({1}, "[1]", False),
        (Decimal("1"), '"1"', False),
        (UUID, f'"{str(UUID)}"', False),
        (datetime.datetime(2018, 1, 1), '"2018-01-01T00:00:00"', False),
        (
            datetime.datetime(2018, 1, 1, tzinfo=UTC),
            '"2018-01-01T00:00:00+00:00"',
            False,
        ),
        (arrow.get("2018-01-01"), '"2018-01-01T00:00:00+00:00"', False),
        (datetime.date(2018, 1, 1), '"2018-01-01"', False),
        (datetime.datetime(2018, 1, 1), "1514764800", True),
        (datetime.datetime(2018, 1, 1, tzinfo=UTC), "1514764800", True),
        (datetime.date(2018, 1, 1), "1514764800", True),
        (arrow.get("2018-01-01"), "1514764800.0", True),
        (HTML(), '"foo"', False),
        (items_view, '{"foo": 1}', False),
    ),
)
def test_dumps(value, expected, date_as_unix_time):
    assert dumps(value, date_as_unix_time=date_as_unix_time) == expected


@pytest.mark.parametrize(
    "values, expected, precision",
    (
        ((1.333, 1.333), "1.33", 2),
        (({1: 1.333}, {1: 1.333}), '{"1": 1.33}', 2),
        (([1.333, 2.333], [1.333, 2.333]), "[1.33, 2.33]", 2),
        (([1.333, {1: 1.333}], [1.333, {1: 1.333}]), '[1.33, {"1": 1.33}]', 2),
        (
            ([1.333, {1: 1.333}, {1.333}], [1.333, {1: 1.333}, {1.333}]),
            '[1.33, {"1": 1.33}, [1.33]]',
            2,
        ),
        ((items_view_float, items_view_float), '{"foo": 1.33}', 2),
        ((items_view_complex, items_view_complex), '{"1": 1.33, "2": {"2": 0.33}}', 2),
        ((HTML(), None), '"foo"', 2),
        (([set()], [set()]), "[[]]", 2),
        (((1.3333, 2.3333), (1.3333, 2.3333)), "[1.33, 2.33]", 2),
        ((({1: 1.33333}, 1.33333), ({1: 1.33333}, 1.33333)), '[{"1": 1.33}, 1.33]', 2),
        (
            (
                [{1: 1.222, 2: [1.333, {1: 1.333}, {3: {3.333}}]}],
                [{1: 1.222, 2: [1.333, {1: 1.333}, {3: {3.333}}]}],
            ),
            '[{"1": 1.22, "2": [1.33, {"1": 1.33}, {"3": [3.33]}]}]',
            2,
        ),
    ),
)
def test_rounding(values, expected, precision):
    before, after = values
    assert dumps(before, precision=precision) == expected
    if after:
        # check if object was not modified by rounding and removing of dict_items
        assert before == after


@pytest.mark.parametrize(
    "values, expected, precision",
    (
        (
            (test_namedtuple, test_namedtuple),
            {"as_object": '{"a": 1.33, "b": 2.33}', "as_list": "[1.33, 2.33]"},
            2,
        ),
        (
            (test_namedtuple_complex, test_namedtuple_complex),
            {
                "as_object": '{"a": 1.33, "b": {"a": 1.33, "b": {"1": 1.33}}}',
                "as_list": '[1.33, [1.33, {"1": 1.33}]]',
            },
            2,
        ),
    ),
)
def test_rounding_tuples(values, expected, precision):
    before, after = values
    if simplejson_dumps:
        # simplejson supports `namedtuple_as_object` param unlike json
        assert dumps(before, precision=precision) == expected["as_object"]
        assert (
            dumps(before, precision=precision, namedtuple_as_object=False)
            == expected["as_list"]
        )
    else:
        assert dumps(before, precision=precision) == expected["as_list"]
    assert before == after


def test_date_as_unix_time_default():
    # When `date_as_unix_time` is not passed, then it is disabled and dates are converted as ISO
    assert dumps(datetime.date(2018, 1, 1)) == '"2018-01-01"'


def test_raw_encoder_with_date_unix_time_default():
    class Foo(object):
        def __repr__(self):
            return "<Foo>"

    # by default `raw_encoder` encodes dates as ISO
    assert dumps(
        {"foo": Foo(), "bar": datetime.date(2018, 1, 1)}, default=raw_encoder
    ) == dumps({"foo": "<Foo>", "bar": "2018-01-01"})


def test_dump_with_default():
    # by default `dumps` encodes dates as ISO
    assert dumps(datetime.date(2018, 1, 1), default=str) == '"2018-01-01"'


def test_dump_with_default_and_date_as_unix_time():
    with pytest.raises(TypeError):
        # `date_as_unix_time` leads to failing when `default` is specified
        dumps(datetime.date(2018, 1, 1), default=str, date_as_unix_time=True)


@pytest.mark.parametrize(
    "value, expected",
    (
        ({1}, [1]),
        (Decimal("1"), "1"),
        (UUID, str(UUID)),
        (datetime.datetime(2018, 1, 1), "2018-01-01T00:00:00"),
        (datetime.datetime(2018, 1, 1, tzinfo=UTC), "2018-01-01T00:00:00+00:00"),
        (datetime.date(2018, 1, 1), "2018-01-01"),
        (HTML(), "foo"),
        (items_view, {"foo": 1}),
    ),
)
def test_dump(tmpdir, value, expected):
    filename = str(tmpdir.join("test_file.json"))
    with open(filename, "w+", encoding="UTF-8") as fp:
        dump(value, fp)
        fp.seek(0, 0)
        assert json_load(fp) == expected


@pytest.mark.skipif(enum is None, reason="Enum is not available")
def test_enum():
    class SomeEnum(enum.Enum):
        red = 1
        green = 2

    assert default_encoder(SomeEnum.red) == "red"


@pytest.mark.skipif(enum is None, reason="Enum is not available")
def test_enum_raw_encoder():
    class SomeEnum(enum.Enum):
        red = 1
        green = 2

    assert raw_encoder(SomeEnum.red) == "red"


def test_class_raw_encoder():
    class Foo(object):
        bar = True  # pylint: disable=C0102

    assert isinstance(raw_encoder(Foo), str)


def test_unknown_raises():
    class Foo(object):
        bar = True  # pylint: disable=C0102

    with pytest.raises(
        TypeError, match="^Object of type Foo is not JSON serializable$"
    ):
        default_encoder(Foo())


@pytest.mark.parametrize(
    "value, expected",
    (
        ({"secret": "FOOO"}, '{"secret": "-- MASKED --"}'),
        ({"booking_token": "FOOO"}, '{"booking_token": "FOOO"}'),
        ({"token": "FOOO"}, '{"token": "-- MASKED --"}'),
        ({"regular_stuff": "FOOO"}, '{"regular_stuff": "FOOO"}'),
        (("regular_stuff", "FOOO"), '["regular_stuff", "FOOO"]'),
    ),
)
def test_masked_json_encoders(value, expected):
    _dumps = simplejson_dumps or json_dumps
    assert _dumps(value, cls=MaskedJSONEncoder) == expected


@pytest.mark.parametrize(
    "dumper, expected",
    (
        (default_encoder, {"attrib": 1}),
        (raw_encoder, {"attrib": 1}),
        (partial(json_dumps, default=default_encoder), '{"attrib": 1}'),
        (partial(json_dumps, default=raw_encoder), '{"attrib": 1}'),
        (partial(json_dumps, cls=KiwiJSONEncoder), '{"attrib": 1}'),
        (partial(json_dumps, cls=MaskedJSONEncoder), '{"attrib": 1}'),
    ),
)
def test_dataclasses(dumper, expected):
    assert dumper(DataclassItem(attrib=1)) == expected  # pylint: disable=not-callable


@pytest.mark.parametrize(
    "dumper, expected",
    (
        (default_encoder, {"attrib": 1}),
        (partial(json_dumps, default=default_encoder), '{"attrib": 1}'),
    ),
)
def test_attrs(dumper, expected):
    assert dumper(AttrsItem(attrib=1)) == expected


@pytest.fixture
def alchemy_session():
    engine = create_engine("sqlite:///:memory:")
    Base.metadata.create_all(engine)

    session = sessionmaker(bind=engine)()

    user = User(name="test")
    session.add(user)
    session.commit()
    return session


def assert_json(value, expected):
    """To simplify tests for Python versions that don't preserve ordering in dicts."""
    encoded = json_dumps(value, default=default_encoder)
    assert loads(encoded) == expected


def test_dictalchemy(alchemy_session):
    instance = alchemy_session.query(User).first()
    assert_json(instance, {"id": 1, "name": "test"})


def test_sqlalchemy_cursor_row(alchemy_session):
    data = alchemy_session.execute("SELECT * FROM users").fetchall()
    assert_json(data, [{"id": 1, "name": "test"}])


def test_no_attrs():
    # Need to re-import
    del sys.modules["kw.json"]
    del sys.modules["kw.json.encode"]
    sys.modules["attr"] = None
    from kw.json import (  # pylint: disable=reimported,import-outside-toplevel
        default_encoder,
    )

    with pytest.raises(
        TypeError, match="Object of type NotAttrsItem is not JSON serializable"
    ):
        default_encoder(NotAttrsItem())


async def get_asyncpg_record(dsn):
    connection = await asyncpg.connect(dsn)
    result = await connection.fetch("SELECT 1 as value")
    await connection.close()
    return result


def test_asyncpg():
    loop = asyncio.get_event_loop()
    result = loop.run_until_complete(get_asyncpg_record(os.getenv("DATABASE_URI")))
    assert json_dumps(result, default=default_encoder) == '[{"value": 1}]'



import calendar
import uuid
from collections import namedtuple
from collections.abc import ItemsView
from decimal import Decimal
from functools import partial

from ._compat import BaseJSONEncoder, _dump, _dumps, enum, simplejson_available
from .utils import mask_dict


def _fail(obj, *args, **kwargs):
    raise TypeError(f"Object of type {obj.__class__.__name__} is not JSON serializable")


try:
    from attr import asdict as attr_asdict
except ImportError:
    attr_asdict = _fail

try:
    from dataclasses import asdict as dc_asdict
except ImportError:
    dc_asdict = _fail


def default_encoder(obj, dict_factory=dict, date_as_unix_time=False):
    if hasattr(obj, "isoformat"):  # date, datetime, arrow
        if date_as_unix_time:
            if obj.__class__.__name__ == "Arrow":
                if callable(obj.timestamp):
                    return obj.timestamp()
                return obj.timestamp
            return calendar.timegm(obj.timetuple())
        return obj.isoformat()

    if isinstance(obj, (Decimal, uuid.UUID)):
        return str(obj)

    if isinstance(obj, set):
        return list(obj)

    if enum is not None and isinstance(obj, enum.Enum):
        return obj.name

    if isinstance(obj, ItemsView):
        return dict_factory(obj)

    if hasattr(obj, "asdict"):
        return dict_factory(obj.asdict().items())

    if hasattr(obj, "__module__") and "sqlalchemy" in obj.__module__:
        if obj.__class__.__name__ == "RowProxy":
            return dict_factory(obj.items())
        if obj.__class__.__name__ == "Row":
            return dict_factory(obj)

    if obj.__class__.__name__ == "Record":  # asyncpg
        return dict_factory(obj)

    if hasattr(obj, "__dataclass_fields__"):  # dataclasses
        return dc_asdict(obj, dict_factory=dict_factory)

    if hasattr(obj, "__attrs_attrs__"):  # attrs
        return attr_asdict(obj, dict_factory=dict_factory)

    if hasattr(obj, "__html__"):
        return str(obj.__html__())

    _fail(obj)


def raw_encoder(obj, date_as_unix_time=False):
    """Return representation of values that are not encodable instead of encoding them."""
    try:
        return default_encoder(
            obj, dict_factory=mask_dict, date_as_unix_time=date_as_unix_time
        )
    except TypeError:
        return repr(obj)


class MaskedJSONEncoder(BaseJSONEncoder):
    def default(self, o):  # pylint: disable=method-hidden
        return default_encoder(o, mask_dict)

    def encode(self, o):
        if isinstance(o, dict):
            o = mask_dict(o)
        return super().encode(o)


class KiwiJSONEncoder(BaseJSONEncoder):
    def default(self, o):  # pylint: disable=method-hidden
        return default_encoder(o)


def modify_kwargs(kwargs):
    # To keep consistent behavior even if simplejson behaves differently by default
    # See #7
    if simplejson_available:
        kwargs.setdefault("use_decimal", False)
    if "default" not in kwargs:
        date_as_unix_time = kwargs.pop("date_as_unix_time", False)
        kwargs["default"] = partial(
            default_encoder, date_as_unix_time=date_as_unix_time
        )


def format_value(value, precision):
    """Format provided value."""
    if isinstance(value, float):
        return round(value, precision)
    if isinstance(value, (list, set)):
        return traverse_iterable(value, precision)
    if (
        isinstance(value, ItemsView)
        or value.__class__.__name__ == "dictionary-itemiterator"
    ):
        return traverse_dict(dict(value), precision)
    if isinstance(value, dict):
        return traverse_dict(value, precision)
    if isinstance(value, tuple):
        if getattr(value, "_fields", False):
            # namedtuple should stay namedtuple - simplejson can format it differently
            # depending on the `namedtuple_as_object` param
            NewNamedtuple = namedtuple(type(value).__name__, value._asdict().keys())
            return NewNamedtuple(**format_value(value._asdict(), precision))
        # convert tuple to list; the tuple would be turned to list in simplejson/json anyways
        return traverse_iterable(list(value), precision)
    return value


def traverse_iterable(iterable, precision):
    """Traverse list or set and round floats."""
    return [format_value(value, precision) for value in iterable]


def traverse_dict(obj, precision):
    """Traverse dictionary and round floats."""
    return {key: format_value(value, precision) for key, value in obj.items()}


def round_floats(args, kwargs):
    data = args[0]
    precision = kwargs.pop("precision", False)
    new_args = list(args)
    if precision is not False:
        data = format_value(data, precision)
        new_args[0] = data
    return new_args


def dumps(*args, **kwargs):
    new_args = round_floats(args, kwargs)
    modify_kwargs(kwargs)
    return _dumps(*new_args, **kwargs)


def dump(*args, **kwargs):
    new_args = round_floats(args, kwargs)
    modify_kwargs(kwargs)
    return _dump(*new_args, **kwargs)



import pytest

from kw.json import mask_dict, mask_dict_factory


@pytest.mark.parametrize(
    "value, expected",
    (
        (None, {}),
        ((("test", 1),), {"test": 1}),
        ([("test", 1), ("testo", 2)], {"test": 1, "testo": 2}),
        ({"test": 1}, {"test": 1}),
        ({"secret": "FOO"}, {"secret": "-- MASKED --"}),
        ({"public_key": "FOO"}, {"public_key": "FOO"}),
    ),
)
def test_mask_dict(value, expected):
    assert mask_dict(value) == expected


@pytest.mark.parametrize(
    "blacklist, whitelist, value, expected",
    (
        (frozenset(), frozenset(), {"secret": "42"}, {"secret": "42"}),
        (
            frozenset(["doom"]),
            frozenset(["doomed"]),
            {"doomed": "we_are", "dooming": "nope"},
            {"doomed": "we_are", "dooming": "-- MASKED --"},
        ),
    ),
)
def test_mask_dict_factory(blacklist, whitelist, value, expected):
    my_mask_dict = mask_dict_factory(blacklist=blacklist, whitelist=whitelist)
    assert my_mask_dict(value) == expected



DEFAULT_PLACEHOLDER = "-- MASKED --"
DEFAULT_BLACKLIST = frozenset(("secret", "token", "password", "key", "zoozappkey"))
DEFAULT_WHITELIST = frozenset(("booking_token", "public_key", "idempotency_key"))


def mask_dict_factory(
    placeholder=DEFAULT_PLACEHOLDER,
    blacklist=DEFAULT_BLACKLIST,
    whitelist=DEFAULT_WHITELIST,
):
    def mask_dict(pairs):
        """Return a dict with dangerous looking key/value pairs masked."""
        if pairs is None:
            return {}

        if isinstance(pairs, dict):
            items = pairs.items()
        else:
            items = pairs

        return {
            key: (
                placeholder
                if key.lower() not in whitelist
                and any(word in key.lower() for word in blacklist)
                else value
            )
            for key, value in items
        }

    return mask_dict


mask_dict = mask_dict_factory()



from decimal import Decimal

import pytest

from kw.json import loads
from kw.json.exceptions import KiwiJsonError

try:
    from simplejson import loads as simplejson_loads
except ImportError:
    simplejson_loads = None


@pytest.mark.skipif(
    simplejson_loads is None, reason="Decimal encoding with simplejson only"
)
@pytest.mark.parametrize(
    "value, expected",
    [
        ('"1"', "1"),
        ("1", Decimal("1")),
        ("-1", Decimal("-1")),
        ("1.0", Decimal("1.0")),
        ("0.123456789123456789", Decimal("0.123456789123456789")),
        ('{"num": 0.123456789123456789}', {"num": Decimal("0.123456789123456789")}),
    ],
    # ids=["str", "int", "neg_int", "float", "long_float", "object"]
)
def test_simplejson_encoder_with_decimal(value, expected):
    assert loads(value, use_decimal=True) == expected


@pytest.mark.skipif(simplejson_loads, reason="Standard json module only")
def test_json_with_use_decimal_argument():
    with pytest.raises(KiwiJsonError, match=r".*Decimal.*"):
        loads("{}", use_decimal=True)



try:
    import enum  # pylint: disable=W0611
except ImportError:
    enum = None

try:
    from simplejson.encoder import (  # pylint: disable=W0611
        JSONEncoder as BaseJSONEncoder,
    )
    from simplejson import dumps as json_dumps  # pylint: disable=W0611
    from simplejson import dump as json_dump  # pylint: disable=W0611
    from simplejson import loads as json_loads  # pylint: disable=W0611
    from simplejson import load as json_load  # pylint: disable=W0611

    simplejson_available = True
except ImportError:
    from json.encoder import JSONEncoder as BaseJSONEncoder  # pylint: disable=W0611
    from json import dumps as json_dumps  # pylint: disable=W0611
    from json import dump as json_dump  # pylint: disable=W0611
    from json import loads as json_loads  # pylint: disable=W0611
    from json import load as json_load  # pylint: disable=W0611

    simplejson_available = False
from .exceptions import KiwiJsonError

from dataclasses import dataclass


@dataclass
class DataclassItem:
    attrib: int


def prevent_unexpected_argument_error(func):
    __use_decimal_error_message = (
        "You can't serialise Decimal as JSON number safely with the standard 'json' module. "
        "Make sure you have 'simplejson' or similar installed."
    )

    def wrapper(*args, **kwargs):
        try:
            result = func(*args, **kwargs)
        except TypeError as err:
            if str(err).endswith(
                "__init__() got an unexpected keyword argument 'use_decimal'"
            ):
                raise KiwiJsonError(  # pylint: disable=W0707
                    __use_decimal_error_message
                )
            raise
        return result

    return wrapper


_dumps = prevent_unexpected_argument_error(json_dumps)
_dump = prevent_unexpected_argument_error(json_dump)
_loads = prevent_unexpected_argument_error(json_loads)
_load = prevent_unexpected_argument_error(json_load)



from __future__ import absolute_import

from ._compat import _load as load
from ._compat import _loads as loads
from .encode import (
    KiwiJSONEncoder,
    MaskedJSONEncoder,
    default_encoder,
    dump,
    dumps,
    raw_encoder,
)
from .flask import JSONExtension
from .utils import (
    DEFAULT_BLACKLIST,
    DEFAULT_PLACEHOLDER,
    DEFAULT_WHITELIST,
    mask_dict,
    mask_dict_factory,
)



from __future__ import absolute_import

from .encode import default_encoder


class JSONExtension(object):
    def __init__(self, app=None, encoder=default_encoder, dict_factory=dict):
        self.app = app
        if app is not None:
            self.init_app(app, encoder, dict_factory)

    def init_app(self, app, encoder=default_encoder, dict_factory=dict):
        from flask import json  # pylint: disable=import-outside-toplevel

        class JSONEncoder(json.JSONEncoder):
            def default(self, o):
                return encoder(o, dict_factory)

        app.json_encoder = JSONEncoder



from setuptools import find_packages, setup

with open("requirements.txt") as f:
    REQUIREMENTS = f.read().splitlines()

with open("test-requirements.txt") as f:
    TEST_REQUIREMENTS = f.read().splitlines()

# read the contents of your README file
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="kiwi-json",
    version="0.10.0",
    url="https://github.com/kiwicom/kiwi-json",
    author="Kiwi.com platform team",
    author_email="platform@kiwi.com",
    packages=find_packages(exclude=["test*"]),
    install_requires=REQUIREMENTS,
    tests_require=TEST_REQUIREMENTS,
    description="DRY JSON encoder.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)
